@@ -44,6 +44,9 @@ class SQLExecute(object):
         # 程序Spool输出句柄
         self.spoolfile = None
 
+        # 程序Echo输出句柄
+        self.echofile = None
+
     def set_Worker_Name(self, p_szWorkerName):
         self.m_Worker_Name = p_szWorkerName
 
@@ -75,6 +78,20 @@ class SQLExecute(object):
          ret_SQLSplitResultsWithComments, ret_SQLHints) = SQLAnalyze(statement)
         for m_nPos in range(0, len(ret_SQLSplitResults)):
             m_raw_sql = ret_SQLSplitResults[m_nPos]                  # 记录原始SQL
+            # 如果当前是在回显一个文件，并且不是echo off，则不再做任何处理
+            if self.echofile is not None and \
+                    not re.match(r'echo\s+off', m_raw_sql, re.IGNORECASE):
+                if self.SQLOptions.get("ECHO").upper() == 'ON' and self.logfile is not None:
+                    click.echo(m_raw_sql, file=self.logfile)
+                if self.SQLOptions.get("ECHO").upper() == 'ON' and self.spoolfile is not None:
+                    click.echo(m_raw_sql, file=self.spoolfile)
+                if self.logger is not None:
+                    if m_raw_sql is not None:
+                        self.logger.info(m_raw_sql)
+                click.echo(m_raw_sql, file=self.echofile)
+                yield None, None, None, None, m_raw_sql
+                continue
+
             sql = m_raw_sql                                          # 当前要被执行的SQL，这个SQL可能被随后的注释或者替换规则改写
             m_CommentSQL = ret_SQLSplitResultsWithComments[m_nPos]   # 记录带有注释信息的SQL
             m_SQLHint = ret_SQLHints[m_nPos]                         # SQL提示信息，其中Feature,SQLID,SQLGROUP用作日志处理
@@ -158,7 +175,7 @@ class SQLExecute(object):
                     self.logger.info(SQLFormatWithPrefix("Your SQL has been changed to:\n" + sql, 'REWROTED '))
 
             # lastsqlresult.LastSQLResult[X][Y]
-            matchObj = re.search(r"%lastsqlresult.LastSQLResult\[(\d+)\]\[(\d+)\]%",
+            matchObj = re.search(r"%lastsqlresult.LastSQLResult\[(\d+)]\[(\d+)]%",
                                  sql, re.IGNORECASE | re.DOTALL)
             if matchObj:
                 m_Searched = matchObj.group(0)
@@ -185,19 +202,21 @@ class SQLExecute(object):
             # ${var}
             bMatched = False
             while True:
-                matchObj = re.search(r"(.*)\${([0-9A-Za-z_\-]+)\}(.*)?",
+                matchObj = re.search(r"(.*)\${([0-9A-Za-z_\-]+)}(.*)?",
                                      sql, re.IGNORECASE | re.DOTALL)
                 if matchObj:
                     bMatched = True
-                    m_VarName = matchObj.group(2)
-                    if not m_VarName.startswith('@'):
-                        m_VarName = '@' + m_VarName
+                    m_VarName = str(matchObj.group(2))
                     m_VarValue = self.SQLOptions.get(m_VarName)
                     if m_VarValue is not None:
                         sql = matchObj.group(1) + m_VarValue + matchObj.group(3)
-                    else:
-                        # 没有定义这个变量，在SQL中把这个变量所对应的位置替换为#UNDEFINE_VAR#来避免死循环
-                        sql = matchObj.group(1) + '#UNDEFINE_VAR#' + matchObj.group(3)
+                        continue
+                    m_VarValue = self.SQLOptions.get('@' + m_VarName)
+                    if m_VarValue is not None:
+                        sql = matchObj.group(1) + m_VarValue + matchObj.group(3)
+                        continue
+                    # 没有定义这个变量，在SQL中把这个变量所对应的位置替换为#UNDEFINE_VAR#来避免死循环
+                    sql = matchObj.group(1) + '#UNDEFINE_VAR#' + matchObj.group(3)
                 else:
                     break
             if bMatched:
@@ -215,7 +234,7 @@ class SQLExecute(object):
 
             # 执行SQL
             try:
-                # 首先假设这是一个特殊命令
+                # 首先尝试这是一个特殊命令，如果返回CommandNotFound，则认为其是一个标准SQL
                 for result in execute(cur, sql):
                     yield result
             except CommandNotFound:
@@ -264,7 +283,10 @@ class SQLExecute(object):
                                     for j in range(0, len(result) - i):
                                         if str(result[j]) > str(result[j + 1]):
                                             result[j], result[j + 1] = result[j + 1], result[j]
-                            yield title, result, headers, columntypes, status
+                            if self.SQLOptions.get('TERMOUT').upper() != 'OFF':
+                                yield title, result, headers, columntypes, status
+                            else:
+                                yield title, [], headers, columntypes, status
                             if not m_FetchStatus:
                                 break
 
@@ -312,6 +334,7 @@ class SQLExecute(object):
                                 str(e).find("SQLSyntaxErrorException") != -1 or
                                 str(e).find("SQLException") != -1 or
                                 str(e).find("SQLDataException") != -1 or
+                                str(e).find("jdbc.exceptions") != -1 or
                                 str(e).find("SQLTransactionRollbackException") != -1 or
                                 str(e).find("SQLTransientConnectionException") != -1 or
                                 str(e).find('time data') != -1
@@ -350,7 +373,6 @@ class SQLExecute(object):
                 if m_TimeLimit < (end - start) * 1000:
                     raise SQLCliException('LinkoopSQL-Err:: ' 
                                           'SQL elapsed time [' + m_TimeCost + '] over limit [' + str(m_TimeLimit) + ']')
-                    # yield None, None, None, None, 'SQL elapsed time over limit [' + str(m_TimeLimit) + ']'
 
             if self.SQLOptions.get('TIMING').upper() == 'ON':
                 if sql.strip().upper() not in ('EXIT', 'QUIT'):
@@ -373,76 +395,75 @@ class SQLExecute(object):
             status = "{0} row{1} selected."
             m_arraysize = int(self.SQLOptions.get("ARRAYSIZE"))
             rowset = list(cursor.fetchmany(m_arraysize))
-            if self.SQLOptions.get('TERMOUT').upper() != 'OFF':
-                for row in rowset:
-                    m_row = []
-                    # 记录字段类型
-                    if len(columntypes) == 0:
-                        for column in row:
-                            columntypes.append(type(column))
+            for row in rowset:
+                m_row = []
+                # 记录字段类型
+                if len(columntypes) == 0:
                     for column in row:
-                        if str(type(column)).upper().find('OBJECT[]') != -1:
-                            m_ColumnValue = "STRUCTURE("
-                            for m_nPos in range(0, len(column)):
-                                m_ColumnType = str(type(column[m_nPos]))
-                                if m_nPos == 0:
-                                    if m_ColumnType.upper().find('STR') != -1:
-                                        m_ColumnValue = m_ColumnValue + "'" + str(column[m_nPos]) + "'"
-                                    elif m_ColumnType.upper().find('SQLDATE') != -1:
-                                        m_ColumnValue = m_ColumnValue + "DATE'" + str(column[m_nPos]) + "'"
-                                    elif str(type(column)).upper().find("FLOAT") != -1:
-                                        m_ColumnValue = m_ColumnValue + \
-                                                        self.SQLOptions.get("FLOAT_FORMAT") % column[m_nPos]
-                                    elif str(type(column)).upper().find("DOUBLE") != -1:
-                                        m_ColumnValue = m_ColumnValue + \
-                                                        self.SQLOptions.get("DOUBLE_FORMAT") % column[m_nPos]
-                                    else:
-                                        m_ColumnValue = m_ColumnValue + str(column[m_nPos])
+                        columntypes.append(type(column))
+                for column in row:
+                    if str(type(column)).upper().find('OBJECT[]') != -1:
+                        m_ColumnValue = "STRUCTURE("
+                        for m_nPos in range(0, len(column)):
+                            m_ColumnType = str(type(column[m_nPos]))
+                            if m_nPos == 0:
+                                if m_ColumnType.upper().find('STR') != -1:
+                                    m_ColumnValue = m_ColumnValue + "'" + str(column[m_nPos]) + "'"
+                                elif m_ColumnType.upper().find('SQLDATE') != -1:
+                                    m_ColumnValue = m_ColumnValue + "DATE'" + str(column[m_nPos]) + "'"
+                                elif str(type(column)).upper().find("FLOAT") != -1:
+                                    m_ColumnValue = m_ColumnValue + \
+                                                    self.SQLOptions.get("FLOAT_FORMAT") % column[m_nPos]
+                                elif str(type(column)).upper().find("DOUBLE") != -1:
+                                    m_ColumnValue = m_ColumnValue + \
+                                                    self.SQLOptions.get("DOUBLE_FORMAT") % column[m_nPos]
                                 else:
-                                    if m_ColumnType.upper().find('STR') != -1:
-                                        m_ColumnValue = m_ColumnValue + ",'" + str(column[m_nPos]) + "'"
-                                    elif m_ColumnType.upper().find('SQLDATE') != -1:
-                                        m_ColumnValue = m_ColumnValue + ",DATE'" + str(column[m_nPos]) + "'"
-                                    elif str(type(column)).upper().find("FLOAT") != -1:
-                                        m_ColumnValue = m_ColumnValue + "," + \
-                                                        self.SQLOptions.get("FLOAT_FORMAT") % column[m_nPos]
-                                    elif str(type(column)).upper().find("DOUBLE") != -1:
-                                        m_ColumnValue = m_ColumnValue + "," + \
-                                                        self.SQLOptions.get("DOUBLE_FORMAT") % column[m_nPos]
-                                    else:
-                                        m_ColumnValue = m_ColumnValue + "," + str(column[m_nPos])
-                            m_ColumnValue = m_ColumnValue + ")"
-                            m_row.append(m_ColumnValue)
-                        elif str(type(column)).upper().find('JDBCCLOBCLIENT') != -1:
-                            m_Length = column.length()
-                            m_ColumnValue = column.getSubString(1, int(self.SQLOptions.get("LOB_LENGTH")))
-                            if m_Length > int(self.SQLOptions.get("LOB_LENGTH")):
-                                m_ColumnValue = m_ColumnValue + "..."
-                            m_row.append(m_ColumnValue)
-                        elif str(type(column)).upper().find('JDBCBLOBCLIENT') != -1:
-                            # 对于二进制数据，用16进制数来显示
-                            # 2: 意思是去掉Hex前面的0x字样
-                            m_Length = column.length()
-                            m_ColumnValue = "".join([hex(int(i))[2:]
-                                                     for i in column.getBytes(1,
-                                                                              int(self.SQLOptions.get("LOB_LENGTH")))])
-                            if m_Length > int(self.SQLOptions.get("LOB_LENGTH")):
-                                m_ColumnValue = m_ColumnValue + "..."
-                            m_row.append(m_ColumnValue)
-                        elif str(type(column)).upper().find("FLOAT") != -1:
-                            m_row.append(self.SQLOptions.get("FLOAT_FORMAT") % column)
-                        elif str(type(column)).upper().find("DOUBLE") != -1:
-                            m_row.append(self.SQLOptions.get("DOUBLE_FORMAT") % column)
-                        elif str(type(column)).upper().find('STR') != -1:
-                            # 对于二进制数据，其末尾用0x00表示，这里进行截断
-                            m_0x00Start = column.find(chr(0))
-                            if m_0x00Start != -1:
-                                column = column[0:m_0x00Start]
-                            m_row.append(column)
-                        else:
-                            m_row.append(column)
-                    m_row = tuple(m_row)
-                    result.append(m_row)
+                                    m_ColumnValue = m_ColumnValue + str(column[m_nPos])
+                            else:
+                                if m_ColumnType.upper().find('STR') != -1:
+                                    m_ColumnValue = m_ColumnValue + ",'" + str(column[m_nPos]) + "'"
+                                elif m_ColumnType.upper().find('SQLDATE') != -1:
+                                    m_ColumnValue = m_ColumnValue + ",DATE'" + str(column[m_nPos]) + "'"
+                                elif str(type(column)).upper().find("FLOAT") != -1:
+                                    m_ColumnValue = m_ColumnValue + "," + \
+                                                    self.SQLOptions.get("FLOAT_FORMAT") % column[m_nPos]
+                                elif str(type(column)).upper().find("DOUBLE") != -1:
+                                    m_ColumnValue = m_ColumnValue + "," + \
+                                                    self.SQLOptions.get("DOUBLE_FORMAT") % column[m_nPos]
+                                else:
+                                    m_ColumnValue = m_ColumnValue + "," + str(column[m_nPos])
+                        m_ColumnValue = m_ColumnValue + ")"
+                        m_row.append(m_ColumnValue)
+                    elif str(type(column)).upper().find('JDBCCLOBCLIENT') != -1:
+                        m_Length = column.length()
+                        m_ColumnValue = column.getSubString(1, int(self.SQLOptions.get("LOB_LENGTH")))
+                        if m_Length > int(self.SQLOptions.get("LOB_LENGTH")):
+                            m_ColumnValue = m_ColumnValue + "..."
+                        m_row.append(m_ColumnValue)
+                    elif str(type(column)).upper().find('JDBCBLOBCLIENT') != -1:
+                        # 对于二进制数据，用16进制数来显示
+                        # 2: 意思是去掉Hex前面的0x字样
+                        m_Length = column.length()
+                        m_ColumnValue = "".join([hex(int(i))[2:]
+                                                 for i in column.getBytes(1,
+                                                                          int(self.SQLOptions.get("LOB_LENGTH")))])
+                        if m_Length > int(self.SQLOptions.get("LOB_LENGTH")):
+                            m_ColumnValue = m_ColumnValue + "..."
+                        m_row.append(m_ColumnValue)
+                    elif str(type(column)).upper().find("FLOAT") != -1:
+                        m_row.append(self.SQLOptions.get("FLOAT_FORMAT") % column)
+                    elif str(type(column)).upper().find("DOUBLE") != -1:
+                        m_row.append(self.SQLOptions.get("DOUBLE_FORMAT") % column)
+                    elif str(type(column)).upper().find('STR') != -1:
+                        # 对于二进制数据，其末尾用0x00表示，这里进行截断
+                        m_0x00Start = column.find(chr(0))
+                        if m_0x00Start != -1:
+                            column = column[0:m_0x00Start]
+                        m_row.append(column)
+                    else:
+                        m_row.append(column)
+                m_row = tuple(m_row)
+                result.append(m_row)
             rowcount = rowcount + len(rowset)
             if len(rowset) < m_arraysize:
                 # 已经没有什么可以取的了, 游标结束
@@ -462,10 +483,7 @@ class SQLExecute(object):
             status = status.format(rowcount, "" if rowcount == 1 else "s")
         else:
             status = None
-        if self.SQLOptions.get('TERMOUT').upper() == 'OFF':
-            return title, [], headers, columntypes, status, m_FetchStatus, rowcount
-        else:
-            return title, result, headers, columntypes, status, m_FetchStatus, rowcount
+        return title, result, headers, columntypes, status, m_FetchStatus, rowcount
 
     # 记录PERF信息，
     def Log_Perf(self, p_SQLResult):
